// irfile.hpp

#ifndef IRFILE_HPP
#define IRFILE_HPP

class IR_FILE_BASE;
class IR_DATA;


class IR_FILE_BASE {

   public:

      #define MAX_CODENAME    512

      IR_FILE_BASE(
         char     *ident_in);

      virtual ~IR_FILE_BASE();

      // Clear the existing file structure
      int   Clear();

      // Read in the specified file (text format)
      virtual int   Read(
         char     *filename) = 0;

      // Write the specified file (text format)
      virtual int   Write(
         char     *filename) = 0;

      // Retrieve IR Data given "CodeName" within file
      IR_DATA *IRFromName(
         char     *codename);

      IR_DATA *IRFromIdx(
         int      idx);

      // Insert new "CodeName" into file using HexStrings. Freq Optional
      int   IRInsert(
         char     *codename, 
         IR_DATA  *data,
         int      idx = -1); 

      // Remove a specified "CodeName" from file
      int   IRRemove(
         char     *codename);

      int   IsIdent(
         char     *ident_in);

      int   GetMaxEntries()      { return MAX_CODENAME; }

   protected   :

      int   IRFind(
         char     *codename);

      int            irdata_alloc;
      IR_DATA        **irdata;

      char           ident[33];

};

class IR_DATA {

   #define MAX_BURSTPAIR   128
   
   struct IR_DS {
      int               count;                     // Count of Burst Pairs
      unsigned long     on_us[MAX_BURSTPAIR];      // On times in us
      unsigned long     off_us[MAX_BURSTPAIR];     // Off times in us
   };

   public:

      enum BP_TYPE {
         BP_TYPE_INITIAL,
         BP_TYPE_REPEAT
      };

      IR_DATA(
         char     *codename,
         unsigned long freq_hz,
         char     *comment = NULL);

      IR_DATA(
         IR_DATA  *data);

      ~IR_DATA();

      IR_DATA *operator=(IR_DATA &);

      int   IsCodename(
         char     *codename);

      int   AddBurstPair(
         BP_TYPE        type,
         unsigned long  on_us,
         unsigned long  off_us);

      int   GetBurstPair(
         BP_TYPE        type,
         unsigned long  *on_us,
         unsigned long  *off_us,
         int            idx);

      int   Merge(
         BP_TYPE     type_dest,
         BP_TYPE     type_src,
         IR_DATA     *data_src);

      unsigned long GetFreq()    { return freq_hz;    }
      unsigned long SetFreq(
         unsigned long  freq)    { return (freq_hz = freq); }

      char  *GetCodename()       { return codename;   }   
      char  *GetComment()        { return comment;    }

      int   GetBurstPairCount(
         BP_TYPE     type);

   private:
   
      char              codename[17];
      char              comment[33];
      unsigned long     freq_hz;
      IR_DS             ir_initial;
      IR_DS             ir_repeat;
};

#endif

