// lir2ir.cpp

#ifdef __USAGE
%C - C-Max LIR to IR text format converter

   -s <filename>     Source .LIR file to convert
   -d <filename>     Destination .IR file. This file will be overwritten

   -M <max>          Maximum number of entries to process from the LIR
                     file. Default is to process all entries in the
                     file (currently 512)
   -Z <debug>        Debug level. 20 is a lot, 30 is too much

File extensions must be specified.

Written by Jay Hogg 04Feb2000.
This program and related source are free for distribution in
a non-commercial environment. No warranties given or implied.
Use at your own risk.

#endif

#include <stdio.h>
#include <stdlib.h>

#ifdef WIN32
   #include "getopt.h"
#else
   #include <unistd.h>
#endif

#include "irfile.hpp"
#include "irfile-lir.hpp"
#include "irfile-ir.hpp"
#include "version.hpp"

#include "trace.hpp"

/* ----------------------------------------------------------------------- */

int main(int argc, char ** argv)
{
   IR_FILE_LIR    *file_lir;
   IR_FILE_IR     *file_ir;

   int      i;
   char     *opt_srcfile   = NULL;
   int      opt_maxread    = -1;
   char     *opt_dstfile   = NULL;
   int      opt;

   printf("IR2LIR Conversion. Version %s\n", VERSION);
   
   // ****** Handle options
   while ((opt = getopt(argc, argv, "s:d:Z:M:")) != -1) {
      switch (opt) {
         case 's':   opt_srcfile = optarg;         break;
         case 'd':   opt_dstfile = optarg;         break;
         case 'M':   opt_maxread = atoi(optarg);   break;
         case 'Z':   TRACE::SetGlobalLevel(atoi(optarg)); break;
      }
   }

   if (!opt_srcfile || !opt_dstfile) {
      printf("Both source (-s) and destination (-d) files must be specified\n");
      return 1;
   }

   // ****** Read the Ocelot File
   file_lir = new IR_FILE_LIR(opt_srcfile);
   file_lir->Read(opt_srcfile, opt_maxread);
   
   // ****** Create a clean file
   file_ir = new IR_FILE_IR(opt_srcfile);

   // ****** Copy all the Ocelot entries over
   for (i=0; i<file_lir->GetMaxEntries(); i++) {
      IR_DATA *ir;
      if ((ir = file_lir->IRFromIdx(i)) != NULL) {
         IR_DATA *ir_copy = new IR_DATA(ir);
         file_ir->IRInsert(ir->GetCodename(), ir_copy);
      }
   }

   // ****** Write the "standard" format IR file
   file_ir->Write(opt_dstfile);

   delete file_ir;
   delete file_lir;

   return 0;
}

/* ----------------------------------------------------------------------- */


