// trace.cpp

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "trace.hpp"

/* ----------------------------------------------------------------------- */

unsigned long TRACE::global_mask = 0xffffffff;
int TRACE::global_level = 0;

/* ----------------------------------------------------------------------- */

TRACE::TRACE(int level_in, char *prefix_in, unsigned long mask_in)
{
   prefix   = prefix_in;
   mask     = mask_in;
   level    = level_in;
   write(level_in, "(enter)");
}

/* ----------------------------------------------------------------------- */

TRACE::~TRACE()
{
   write(level, "(leave)");
}

/* ----------------------------------------------------------------------- */

void TRACE::write(int level_in, char *msg, ...)
{
   if (level_in == 0)
      level_in = level;

   if (level_in > -1) {

      if (mask && ((mask & global_mask) == 0))
         return;

      if (level_in > global_level)
         return;
   }

   printf("%s: ", prefix);

   if (msg) {
      va_list  arglist;
      va_start(arglist,msg);
      vprintf(msg, arglist);
      va_end(arglist);
   }

   puts("");
   fflush(stdout);

}

/* ----------------------------------------------------------------------- */

int TRACE::ret_int(int ret)
{
   write(0, "return(%i)", ret);

   return ret;
}

/* ----------------------------------------------------------------------- */

